<?php

namespace Service;

use Util\OpensslCrypt;
use Doctrine\DBAL\DriverManager;
use Zeedhi\Framework\DataSource\FilterCriteria;
use \Service\Manager\Mongo\Manager;
use Controller\Exception;
use Util\Utilities;
use Util\InstanceProvider;
use Zeedhi\Framework\DataSource\DataSet;

class ConnectionService {

	/** @var \Service\Manager\Mongo\Manager $organizationManager */
	protected $organizationManager;
	/** @var \Service\SocketClient $socketClient */
	protected $socketClient;
	/** @var \Service\Manager\Mongo\Manager $organizationManager */
	protected $defaultOrgManager;

	public function __construct (Manager $organizationManager, \Service\SocketClient $socketClient, Manager $defaultOrgManager) {
		$this->organizationManager = $organizationManager;
		$this->socketClient = $socketClient;
		$this->defaultOrgManager = $defaultOrgManager;
	}

	public function getConnectionParams ($connectionId = null) {
		$conditions = array('operator' => '=');
		$conditions['columnName'] = !is_null($connectionId) ? '_id' : 'main';
		$conditions['value'] = !is_null($connectionId) ? $connectionId : true;

		$filterCriteria = new FilterCriteria('connection', array($conditions));
		$dataSet = $this->organizationManager->findBy($filterCriteria);
		$connectionParams = $dataSet->getRows();

		if(!empty($connectionParams)) {
			return $connectionParams[0];
		} else {
			throw Exception::connectionNotFound();
		}
	}
	
	public function find($dataSourceName, $params, $decodeBase64 = true) {
		return $this->organizationManager->findNext($dataSourceName, $params, $decodeBase64);
	}

	public function getCustomConnection ($connectionId = null) {
		$eventManager = new \Doctrine\Common\EventManager();
		$params = $this->getConnectionParams($connectionId);
		$params["charset"] = "AL32UTF8";
		$params["user"] = $params["username"];
		if ($params['driver'] === 'oci8') {
			$params['driverClass'] = '\Zeedhi\Framework\DBAL\Driver\OCI8\Driver';
			if ($params['language'] === 'pt_br') {
			 $eventManager->addEventSubscriber(new \Doctrine\DBAL\Event\Listeners\OracleSessionInit(array(
			 'NLS_DATE_FORMAT' => "DD/MM/YYYY HH24:MI:SS",
				   'NLS_LANGUAGE' => 'BRAZILIAN PORTUGUESE'
			 )));
			} else {
			 $eventManager->addEventSubscriber(new \Doctrine\DBAL\Event\Listeners\OracleSessionInit(array(
			 'NLS_DATE_FORMAT' => "DD/MM/YYYY HH24:MI:SS"
			 )));
			}
			if (isset($params['sessionMode'])) {
			   $params['sessionMode'] = constant($params['sessionMode']);
			}
		}
		return DriverManager::getConnection($params, null, $eventManager);
	}

	public function createDefaultConnection(){
		$row = $this->buildNewConnParams([]);
		$organizationId = Utilities::getOrganizationIdForMongo();
		$row["oracleWalletUse"] = "Y";
		$row["driverClass"] = "oracle.jdbc.driver.OracleDriver";
		$row["dbUrlSsl"] = OpensslCrypt::encrypt("jdbc:oracle:thin:@USR_ORG_$organizationId");
		$row["name"] = "MAIN CONNECTION";
		$row["service"] = true;
		$row["language"] = "pt_br";
		$row["languageLabel"] = "Portuguese (BR)";
		$row["database"] = "Oracle";
		$row["main"] = true;
		$row["dialect"] = "org.hibernate.dialect.OracleDialect";
		$this->organizationManager->persist(new DataSet('connection', [$row]));
	}

	public function checkIfSaas(){
		$organizationId = Utilities::getOrganizationIdForMongo();
		$query = "SELECT IDORGSAAS FROM ORGANIZACAO WHERE NRORG=$organizationId";
		$entityManager = InstanceProvider::getConnectionCustom()->updateVpdConnection($organizationId,true);
        $entityManager = $entityManager ? $entityManager : InstanceProvider::getEntityManager();
		$rows = $entityManager->getConnection()->fetchAll($query);
		return !empty($rows) && !empty($rows[0]["IDORGSAAS"]) && $rows[0]["IDORGSAAS"] == "S";
	}

	public function executeQuery ($connectionParams, $maxWaitTime = null){
		$query = $connectionParams["query"];
		$room = $connectionParams["room"];
		$newConnectionParams = $this->buildNewConnParams($connectionParams);

		$response = $this->socketClient->execQuery($query, $room, $maxWaitTime?: 15, $newConnectionParams);
		$response = empty($response)? array() : $response[0];

		if(isset($response['hasError']) && $response['hasError']){
			$errorMessage = "Failed to execute query";
			if(isset($response["data"]) && isset($response["data"]["response"]) && isset($response["data"]["response"]["sqlException"]) && isset($response["data"]["response"]["sqlException"]["detailMessage"])){
				$errorMessage = $response["data"]["response"]["sqlException"]["detailMessage"];
			} else {
				$errorMessage = $response['error'];
			}
			throw Exception::throwNewException($errorMessage);
		}

		$queryResult = !empty($response['data']) && !empty($response['data']['response'])? $response['data']['response'] : array();
		return $queryResult;
	}

	protected function buildNewConnParams($connectionParams) {
		return array(
			"userName"	  => isset($connectionParams["username"])    ? $connectionParams["username"]    : null,
			"password"	  => isset($connectionParams["password"])    ? $connectionParams["password"]    : null,
			"dbUrl" 	  => isset($connectionParams["dbUrl"])	     ? $connectionParams["dbUrl"]       : null,
            "userNameSsl" => isset($connectionParams["usernameSsl"]) ? $connectionParams["usernameSsl"] : null,
            "passwordSsl" => isset($connectionParams["passwordSsl"]) ? $connectionParams["passwordSsl"] : null,
            "dbUrlSsl" 	  => isset($connectionParams["dbUrlSsl"])	 ? $connectionParams["dbUrlSsl"]    : null,
            "dialect"	  => isset($connectionParams["dialect"])     ? $connectionParams["dialect"]     : null,
			"driverClass" => isset($connectionParams["driverClass"]) ? $connectionParams["driverClass"] : null,
			"oracleWalletUse" => isset($connectionParams["oracleWalletUse"]) ? $connectionParams["oracleWalletUse"] : null
		);
	}

	public function testConnection($connectionParams, $maxWaitTime = null) {
		$room = $connectionParams["room"];
		$query = $connectionParams["query"];
		$newConnectionParams = $this->buildNewConnParams($connectionParams);
		return $this->socketClient->testConnection($room, $newConnectionParams, $query);
	}

	public function getClientRoom($organizationId, $row = null){

		if(isset($row['room'])){
			$clientRoom = $organizationId."_".$row['room'];
		} else {
			$filterCriteria = new FilterCriteria('connection', array(
				array(
					"columnName"  => "organizationId",
					"value"       => array(strval($organizationId),intval($organizationId)),
					"operator"    => "IN")
				)
			);
			$organizationConnectionParams = $this->defaultOrgManager->findBy($filterCriteria)->getRows()[0];
			$clientRoom = isset($organizationConnectionParams["clientRoom"]) ? $organizationConnectionParams["clientRoom"] : $organizationId;
		}
		return $clientRoom;
	}
}
